/*
 * Copyright (C) 2015-present, Ant Financial Services Group
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.alipay.hulu.shared.node.action;

import android.os.Looper;

import com.alipay.hulu.common.tools.CmdTools;
import com.alipay.hulu.common.utils.MiscUtil;
import com.android.permission.rom.RomUtils;

import java.util.Locale;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Created by qiaoruikai on 2018/10/8 8:51 PM.
 */
public class CmdExecutor {
    private ExecutorService executorService;

    private int clickType = OperationExecutor.CLICK_TYPE_ADB_TAP;
    private String touchDevice;
    private float factorX = 1;
    private float factorY = 1;

    /**
     * 通用Click事件
     */
    private static final String NORMAL_CLICK_EVENT = "sendevent %1$s 1 330 1 && " +
            "sendevent %1$s 1 325 1 && sendevent %1$s 3 57 111 && sendevent %1$s 3 53 %2$d && " +
            "sendevent %1$s 3 54 %3$d && sendevent %1$s 0 0 0 && sendevent %1$s 3 57 4294967295 &&" +
            " sendevent %1$s 1 330 0 && sendevent %1$s 1 325 0 && sendevent %1$s 0 0 0";

    /**
     * OPPO的Click事件
     */
    private static final String OPPO_CLICK_EVENT = "sendevent %1$s 3 57 111 &&" +
            " sendevent %1$s 1 330 1 && sendevent %1$s 1 325 1 && sendevent %1$s 3 53 %2$d && " +
            "sendevent %1$s 3 54 %3$d && sendevent %1$s 0 0 0 && sendevent %1$s 3 57 4294967295 && " +
            "sendevent %1$s 1 330 0 &&  sendevent %1$s 1 325 0 &&  sendevent %1$s 0 0 0";

    private static final String SONY_CLICK_EVENT = "sendevent %1$s 3 57 111 && " +
            "sendevent %1$s 3 55 0 && sendevent %1$s 3 53 %2$d && sendevent %1$s 3 54 %3$d && " +
            "sendevent %1$s 3 58 20 && sendevent %1$s 0 2 0 && sendevent %1$s 0 0 0 && " +
            "sendevent %1$s 3 57 4294967295 && sendevent %1$s 0 2 0 && sendevent %1$s 0 0 0";

    private static final String VIVO_CLICK_EVENT = "sendevent %1$s 1 330 1 &&" +
            " sendevent %1$s 1 325 1 && sendevent %1$s 3 47 0 && sendevent %1$s 3 57 111 && " +
            "sendevent %1$s 3 53 %2$d && sendevent %1$s 3 54 %3$d && sendevent %1$s 3 58 20 && " +
            "sendevent %1$s 0 0 0 && sendevent %1$s 3 57 4294967295 && sendevent %1$s 1 330 0 && " +
            "sendevent %1$s 1 325 0 && sendevent %1$s 0 0 0";

    private static final String GOOGLE_CLICK_EVENT = "sendevent %1$s 3 57 111 && " +
            "sendevent %1$s 1 330 1 && sendevent %1$s 1 325 1 && sendevent %1$s 3 53 %2$d && " +
            "sendevent %1$s 3 54 %3$d && sendevent %1$s 3 58 20 && sendevent %1$s 0 0 0 && " +
            "sendevent %1$s 3 57 4294967295 && sendevent %1$s 1 330 0 &&  sendevent %1$s 1 325 0 && " +
            "sendevent %1$s 0 0 0";

    /**
     * 华为的Click事件
     */
    private static final String HUAWEI_CLICK_EVENT = "sendevent %1$s 3 58 10 &&" +
            " sendevent %1$s 3 53 %2$d && sendevent %1$s 3 54 %3$d && sendevent %1$s 3 57 0 && " +
            "sendevent %1$s 0 2 0 && sendevent %1$s 1 330 1 && sendevent %1$s 0 0 0 && " +
            "sendevent %1$s 1 330 0 &&  sendevent %1$s 0 0 0";

    private static final String MEIZU_CLICK_EVENT = "sendevent %1$s 3 57 111 && " +
            "sendevent %1$s 3 58 20 && sendevent %1$s 3 53 %2$d && sendevent %1$s 3 54 %3$d && " +
            "sendevent %1$s 0 2 0 && sendevent %1$s 1 330 1 && sendevent %1$s 0 0 0 && " +
            "sendevent %1$s 1 330 0 && sendevent %1$s 0 0 0";

    public CmdExecutor() {
        this.executorService = new ThreadPoolExecutor(0, Integer.MAX_VALUE,
                0, TimeUnit.MILLISECONDS,
                new SynchronousQueue<Runnable>());
    }

    public void setClickType(int clickType) {
        this.clickType = clickType;
    }

    public void setTouchDevice(String touchDevice) {
        this.touchDevice = touchDevice;
    }

    public void setFactorX(float factorX) {
        this.factorX = factorX;
    }

    public void setFactorY(float factorY) {
        this.factorY = factorY;
    }

    /**
     * 执行adb命令
     * @param cmd
     */
    public void executeCmd(final String cmd) {
        if (Looper.getMainLooper() == Looper.myLooper()) {
            executorService.execute(new Runnable() {
                @Override
                public void run() {
                    CmdTools.execAdbCmd(cmd, 0);
                }
            });
        } else {
            CmdTools.execAdbCmd(cmd, 0);
        }
    }

    /**
     * 执行点击操作
     * @param x
     * @param y
     */
    public void executeClick(int x, int y) {
        String cmd;
        if (clickType == OperationExecutor.CLICK_TYPE_ADB_TAP) {
            cmd = "input tap " + x + " " + y;
            executeCmd(cmd);
        } else {
            int realX = (int) (x / factorX);
            int realY = (int) (y / factorY);

            // 不同厂商device格式有些差异
            if (RomUtils.checkIsHuaweiRom()) {
                cmd = String.format(Locale.CHINA, HUAWEI_CLICK_EVENT, touchDevice, realX, realY);
            } else if (RomUtils.isOppoSystem()) {
                cmd = String.format(Locale.CHINA, OPPO_CLICK_EVENT, touchDevice, realX, realY);
            } else if (RomUtils.isSonySystem()) {
                cmd = String.format(Locale.CHINA, SONY_CLICK_EVENT, touchDevice, realX, realY);
            } else if (RomUtils.isVivoSystem()) {
                cmd = String.format(Locale.CHINA, VIVO_CLICK_EVENT, touchDevice, realX, realY);
            } else if (RomUtils.checkIsMeizuRom()) {
                cmd = String.format(Locale.CHINA, MEIZU_CLICK_EVENT, touchDevice, realX, realY);
            } else if (RomUtils.isGoogleSystem()) {
                cmd = String.format(Locale.CHINA, GOOGLE_CLICK_EVENT, touchDevice, realX, realY);
            } else {
                cmd = String.format(Locale.CHINA, NORMAL_CLICK_EVENT, touchDevice, realX, realY);
            }

            // 这个需要保证同步执行，即使是在主线程上
            executeCmdSync(cmd);

            // 再等500ms，防止没点击完毕
            MiscUtil.sleep(1000);
        }
    }

    /**
     * 异步执行点击
     * @param x
     * @param y
     */
    public void executeClickAsync(int x, int y) {
        String cmd;
        if (clickType == OperationExecutor.CLICK_TYPE_ADB_TAP) {
            cmd = "input tap " + x + " " + y;
        } else {
            int realX = (int) (x / factorX);
            int realY = (int) (y / factorY);
            cmd = String.format(Locale.CHINA,
                    "sendevent %1$s 1 330 1 && sendevent %1$s 3 53 %2$d &&" +
                            " sendevent %1$s 3 54 %3$d && sendevent %1$s 0 0 0 &&" +
                            " sendevent %1$s 1 330 0 &&  sendevent %1$s 0 0 0", touchDevice, realX, realY);
        }

        executeCmdAsync(cmd);
    }

    /**
     * 异步执行adb命令
     * @param cmd
     */
    public void executeCmdAsync(final String cmd) {
        executorService.execute(new Runnable() {
            @Override
            public void run() {
                CmdTools.execAdbCmd(cmd, 0);
            }
        });
    }

    /**
     * 异步执行adb命令
     * @param cmd
     * @param maxTime 最长执行时间
     */
    public void executeCmdAsync(final String cmd, final int maxTime) {
        executorService.execute(new Runnable() {
            @Override
            public void run() {
                CmdTools.execAdbCmd(cmd, maxTime);
            }
        });
    }

    /**
     * 执行adb命令
     * @param cmd
     * @param maxTime 最大执行时间
     */
    public void executeCmd(final String cmd, final int maxTime) {
        if (Looper.getMainLooper() == Looper.myLooper()) {
            executorService.execute(new Runnable() {
                @Override
                public void run() {
                    CmdTools.execAdbCmd(cmd, maxTime);
                }
            });
        } else {
            CmdTools.execAdbCmd(cmd, maxTime);
        }
    }

    /**
     * 同步执行adb命令
     * @param cmd
     */
    public String executeCmdSync(final String cmd) {
        return CmdTools.execAdbCmd(cmd, 0);
    }

    /**
     * 同步执行adb命令
     * @param cmd
     * @param maxTime 最大执行时间
     */
    public String executeCmdSync(final String cmd, int maxTime) {
        return CmdTools.execAdbCmd(cmd, maxTime);
    }

    /**
     * 执行runnable
     * @param runnable
     */
    public void execute(Runnable runnable) {
        executorService.execute(runnable);
    }
}
